@@ -31,7 +31,8 @@ esac
 # -Wl,--gc-sections makes it impossible to check for undefined references
 # for some reason so we need to patch away the "-no" after Configure and before make :/
 # --allow-multiple-definition exists to work around uclibc's pthread static linking bug
-PERL_LDFLAGS="-Wl,--no-gc-sections -Wl,--allow-multiple-definition"
+#PERL_LDFLAGS="-Wl,--no-gc-sections -Wl,--allow-multiple-definition"
+PERL_LDFLAGS=
 PERL_LIBS="-lm -lcrypt" # perl loves to add lotsa crap itself
 
 # some configuration options for modules
@@ -49,6 +50,7 @@ EXTRA_MODULES=""
 
 # overridable functions
 preconfigure()  { : ; }
+patchconfig()   { : ; }
 postconfigure() { : ; }
 postbuild()     { : ; }
 postinstall()   { : ; }
@@ -73,7 +75,7 @@ LC_ALL=C; export LC_ALL # just to be on the safe side
 
 # set version in a way that Makefile.PL can extract
 VERSION=VERSION; eval \
-$VERSION="1.11"
+$VERSION="1.21"
 
 BZ2=bz2
 BZIP2=bzip2
@@ -167,7 +169,7 @@ EOF
 
       mkdir -p unpack
       rm -rf unpack/perl-$PERL_VERSION
-      $BZIP2 -d <perl-$PERL_VERSION.tar.$BZ2 | tar xfC - unpack \
+      $BZIP2 -d <perl-$PERL_VERSION.tar.$BZ2 | ( cd unpack && tar xf - ) \
          || fatal "perl-$PERL_VERSION.tar.$BZ2: error during unpacking"
       chmod -R u+w unpack/perl-$PERL_VERSION
       mv unpack/perl-$PERL_VERSION perl-$PERL_VERSION
@@ -229,7 +231,11 @@ EOF
    grep -q -- -fstack-protector Configure && \
       sedreplace 's/-fstack-protector/-fno-stack-protector/g' Configure
 
-   preconfigure
+   # what did that bloke think
+   grep -q -- usedl=.define hints/darwin.sh && \
+      sedreplace '/^usedl=.define.;$/d' hints/darwin.sh
+
+   preconfigure || fatal "preconfigure hook failed"
 
 #   trace configure \
    sh Configure -Duselargefiles \
@@ -271,6 +277,8 @@ EOF
       s/ *-fno-stack-protector */ /g
    ' config.sh
 
+   patchconfig || fatal "patchconfig hook failed"
+
    sh Configure -S || fatal "Configure -S failed"
 
    postconfigure || fatal "postconfigure hook failed"
@@ -317,6 +325,22 @@ EOF
       cat >"$PERL_PREFIX"/bin/cpan-make-install <<EOF
 "$MAKE" || exit
 
+# patch CPAN::HandleConfig.pm
+HCPM="$PERL_PREFIX"/lib/CPAN/HandleConfig.pm
+case "\$(head -n1 "\$HCPM")" in
+   *CPAN::MyConfig* )
+      ;;
+   * )
+      echo "patching \$HCPM for a safer tomorrow"
+      {
+         echo "use CPAN::MyConfig;"
+         cat "\$HCPM"
+      } >"\$HCPM~"
+      rm -f "\$HCPM"
+      mv "\$HCPM~" "\$HCPM"
+      ;;
+esac
+
 if find blib/arch/auto -type f | grep -q -v .exists; then
    echo Probably an XS module, rebuilding perl
    if "$MAKE" perl; then
@@ -337,7 +361,11 @@ EOF
       # trick CPAN into avoiding ~/.cpan completely
       echo 1 >"$PERL_PREFIX/lib/CPAN/MyConfig.pm"
 
-      "$PERL_PREFIX"/bin/perl -MCPAN -e '
+      # we call cpan with -MCPAN::MyConfig in this script, but
+      # every make install will patch CPAN::HandleConfig.pm to
+      # protect the user.
+
+      "$PERL_PREFIX"/bin/perl -MCPAN::MyConfig -MCPAN -e '
          CPAN::Shell->o (conf => urllist => push => "'"$CPAN"'");
          CPAN::Shell->o (conf => q<cpan_home>, "'"$STATICPERL"'/cpan");
          CPAN::Shell->o (conf => q<init>);
@@ -349,6 +377,7 @@ EOF
          CPAN::Shell->o (conf => q<make_install_make_command>, "'"$PERL_PREFIX"'/bin/cpan-make-install");
          CPAN::Shell->o (conf => q<prerequisites_policy>, q<follow>);
          CPAN::Shell->o (conf => q<build_requires_install_policy>, q<no>);
+         CPAN::Shell->o (conf => q<prefer_installer>, "EUMM");
          CPAN::Shell->o (conf => q<commit>);
       ' || fatal "error while initialising CPAN"
 
@@ -378,7 +407,7 @@ $@
 EOF
 
    for mod in "$@"; do
-      "$PERL_PREFIX"/bin/perl -MCPAN -e 'notest install => "'"$mod"'"' \
+      "$PERL_PREFIX"/bin/perl -MCPAN::MyConfig -MCPAN -e 'notest install => "'"$mod"'"' \
          || fatal "$mod: unable to install from CPAN"
    done
    rm -rf "$STATICPERL/build"
